//---------------------------------------------------------------------------

// This software is Copyright (c) 2015 Embarcadero Technologies, Inc.
// You may only use this software if you are an authorized licensee
// of an Embarcadero developer tools product.
// This software is considered a Redistributable as defined under
// the software license agreement that comes with the Embarcadero Products
// and is subject to that software license agreement.

//---------------------------------------------------------------------------

unit MainFm;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Types3D, FMX.Materials,
  FMX.Forms3D, System.Math, FMX.Graphics, FMX.MaterialSources;

type
  TMainForm = class(TForm3D)
    TextureMaterialSource1: TTextureMaterialSource;
    procedure Form1Render(Sender: TObject; const Context: TContext3D);
    procedure Timer1(Sender: TObject);
    procedure Form3DCreate(Sender: TObject);
    procedure Form3DDestroy(Sender: TObject);
  private
    { Private declarations }
    FVertexBuffer: TVertexBuffer;
    FIndexBuffer: TIndexBuffer;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
{$R *.fmx}
{$R *.LgXhdpiPh.fmx ANDROID}

uses
  System.Math.Vectors, System.RTLConsts;

procedure TMainForm.Form3DCreate(Sender: TObject);
begin
  FVertexBuffer := TVertexBuffer.Create([TVertexFormat.Vertex, TVertexFormat.TexCoord0], 4);
  FVertexBuffer.Vertices[0] := Point3D(-10.0, 0.0, -5.0);
  FVertexBuffer.Vertices[1] := Point3D(10.0, 0, -5.0);
  FVertexBuffer.Vertices[2] := Point3D(10.0, 0, 5.0);
  FVertexBuffer.Vertices[3] := Point3D(-10.0, 0, 5.0);

  FVertexBuffer.TexCoord0[0] := TPointF.Create(0.0, 0.0);
  FVertexBuffer.TexCoord0[1] := TPointF.Create(1.0, 0.0);
  FVertexBuffer.TexCoord0[2] := TPointF.Create(1.0, 1.0);
  FVertexBuffer.TexCoord0[3] := TPointF.Create(0.0, 1.0);

  FIndexBuffer := TIndexBuffer.Create(6);
  FIndexBuffer[0] := 0;
  FIndexBuffer[1] := 1;
  FIndexBuffer[2] := 3;
  FIndexBuffer[3] := 3;
  FIndexBuffer[4] := 1;
  FIndexBuffer[5] := 2;
end;

procedure TMainForm.Form3DDestroy(Sender: TObject);
begin
  FIndexBuffer.Free;
  FVertexBuffer.Free;
end;

procedure TMainForm.Form1Render(Sender: TObject; const Context: TContext3D);
begin
  Context.SetMatrix(TMatrix3D.CreateRotation(TPoint3D.Create(1.0, 1.0, 1.0), DegToRad(Tag)));
  Context.SetContextState(TContextState.csAllFace);
  Context.DrawTriangles(FVertexBuffer, FIndexBuffer, TextureMaterialSource1.Material, 1.0);
end;

procedure TMainForm.Timer1(Sender: TObject);
begin
  Tag := Tag + 1;
  Invalidate;
end;

end.
